/*
----------------------------------------------------------------------
  C++ classes designed for photoelectron diffraction software package
  Yufeng Chen, Michel A. Van Hove
  Lawrence Berkeley National Laboratory (LBNL), Berkeley, CA 94720
  Copyright (c) Van Hove Group 1997. All rights reserved.
----------------------------------------------------------------------

  Yufeng Chen, LBNL MS 2-100, 1 Cyclotron Road, Berkeley, CA 94720
  Phone (510) 486-4581 Fax (510) 486-5530 Email. ychen@LBL.gov

  M. A. Van Hove, LBNL MS 66-100, 1 Cyclotron Road, Berkeley, CA 94720
  Phone (510) 486-6160 Fax (510) 486-4995 Email. MAVanhove@LBL.gov
----------------------------------------------------------------------

List of classes

1. Gausquad
Description:
  calculate two-electron repulsion or exchange integrals using
  Rys quadrature method.
  J. Rys, M. Dupuis, and H.F. King, Computation of electron
  repulsion integrals using the Rys quadrature method,
  J. Computational Chem. 4 (2), 154-157 (1983)

  The evaluation of the electron-repulsion integrals
  (ij||kl)=2(rho/pi)(1/2)sum(Ix Iy Iz W)=sum(Ix Iy Iz')
  where Iz'=2(rho/pi)(1/2)*W
  This object calculate Ix, Iy or Iz'

Constructors and Destructor:
  Gausquad() --- constructor
Member Functions:
  void init() --- initialization, must do before any other calls
  int getlength() --- return memory size needed by an object
  int export(char *dest,int length=0) --- export an object contents
    into memory buffer dest
  int import(char *source,int length=0) --- import an object contents
    from memory buffer source
  float getmemory() --- return memory used by an object
  int loadparas(int waiver,int *narray,float *xarray,float *karray,
    Ryspoly *ryspoly)
    --- load parameters, primitive gaussian = x(nx)y(ny)z(nz)exp(-k*r2)
    narray is address of 12 (nx ny nz) numbers for 4 primitive each
    (in order nxa nxb nxc nxd nya nyb nyc nyd nza nzb nzc nzd)
    xarray is address of 12 (x y z) centers for 4 primitive each
    (in order xa xb xc xd ya yb yc yd za zb zc zd)
    karray is address of 4 exponential indecies for 4 primitive each
    (in order ka kb kc kd)
    ryspoly is the address of a Rys polynomial object
    waiver is a switch to waive Ix, Iy or Iz calculation due to the
      symmetry. bit-0 1 2 from right are waiver for Ix, Iy and Iz
      respectively
  float fquadint() --- return the sum of integrals of 4 primitive
    Gaussian base functions

Private Member Functions:
  int makeintegral(int waiver,int *narray,float *xarray,float *karray)
    --- make sum of integrals of 4 primitive Gaussian base functions
    waiver is a switch to waive Ix, Iy or Iz calculation due to the
      symmetry. bit-0 1 2 from right are waiver for Ix, Iy and Iz
      respectively
  int makexelem(int nx,int *narray,float *xarray,float *karray) ---
    make Ix, Iy, or Iz

----------------------------------------------------------------------
*/

#ifndef __GAUSQUAD_H
#define __GAUSQUAD_H

#include "ryspoly.h"

class Gausquad
{ private:
    int nt,ndima,ndimb,ndimc,ndimd,ndims,ndimt,error;
    float *talpha,*walpha,*ualpha,*quadint;
  public:
    Gausquad();
    ~Gausquad();
    void init();
    int getlength();
    int export(char *dest,int length=0);
    int import(char *source,int length=0);
    float getmemory();
    int loadparas(int waiver,int *narray,float *xarray,float *karray,
      Ryspoly *ryspoly);
    float fquadint();
  private:
    int makeintegral(int waiver,int *narray,float *xarray,
      float *karray);
    int makexelem(int nx,int *narray,float *xarray,float *karray);
};

#endif //__GAUSQUAD_H

