/*
----------------------------------------------------------------------
  C++ classes designed for photoelectron diffraction software package
  Yufeng Chen, Michel A. Van Hove
  Lawrence Berkeley National Laboratory (LBNL), Berkeley, CA 94720
  Copyright (c) Van Hove Group 1997. All rights reserved.
----------------------------------------------------------------------

  Yufeng Chen, LBNL MS 2-100, 1 Cyclotron Road, Berkeley, CA 94720
  Phone (510) 486-4581 Fax (510) 486-5530 Email. ychen@LBL.gov

  M. A. Van Hove, LBNL MS 66-100, 1 Cyclotron Road, Berkeley, CA 94720
  Phone (510) 486-6160 Fax (510) 486-4995 Email. MAVanhove@LBL.gov
----------------------------------------------------------------------

List of classes

1. Ryspoly
Description:
  calculate roots and quadrature weight factors of Rys polynomial.
  H.F. King and M. Dupuis, Numerical integration using rys polynomials,
  J. Computational Physics 21, 144-165 (1976)
Constructors and Destructor:
  Ryspoly() --- constructor
Member Functions:
  void init() --- initialization, must do before any other calls
  int getlength() --- return memory size needed by an object
  int export(char *dest,int length=0) --- export an object contents
    into memory buffer dest
  int import(char *source,int length=0) --- import an object contents
    from memory buffer source
  float getmemory() --- return memory used by an object
  int makeroots(int anrys) --- calculate all roots and quadrature
    weight factors for up to anrys order
  double getualpha(int mrys,int mroot,double xa) --- return ualpha
    (ualpha=talpha*talpha/(1-talpha*talpha), where talpha is root,
    mrys is order of Rys polynomial, mroot (0 - mrys-1) is number
    of the specified root, xa is parameter of the Rys polynomial
  double getwalpha(int mrys,int mroot,double xa) --- return quadrature
    weight factor walpha, where mrys is order of Rys polynomial,
    mroot (0 - mrys-1) is number of the specified root, xa is
    parameter of the Rys polynomial
  double gettalpha(int mrys,int mroot,double xa) --- return zero
    root talpha, where mrys is order of Rys polynomial,
    mroot (0 - mrys-1) is number of the specified root, xa is
    parameter of the Rys polynomial
  int getalphas(int mrys,double xa,double *xtalpha,
    double *xwalpha,double *xualpha) --- calculate talpha, walpha and
    ualpha without any error check
  int getalphas(int mrys,float xa,float *xtalpha,
    float *xwalpha,float *xualpha) --- calculate talpha, walpha and
    ualpha without any error check
  int saveroots(char *filename,char *usermessage) --- save roots and
    weight factors into a file with name of filename and user message
    in usermessage
Private Member Functions:
  int makelement() --- make all Rys elements,
    In(x)=Integral(t=0 - 1)(t^2n exp(-xt2)) dt
  int makesubelement(double xa) --- make Rys elements for parameter xa
  double getelement(int bnrys,double xa) --- return element of bnrys
    order Rys polynomial of parameter xa
  double fryspoly(int mrys,double xt) --- return value of the
    mrys order of Rys polynomial of parameter xt
  double asymptotic(int k,int ma,double *xalpha) --- return root
    or weight factor through asymptotical formula, where k is number
    of a specific point, ma is number of specific root or weight
    factor, and xalpha is roots or factors buffer talpha or walpha

----------------------------------------------------------------------
*/

#ifndef __RYSPOLY_H
#define __RYSPOLY_H

#include "jobtime.h"

class Ryspoly
{ private:
    int datatype,ndata,nelem,nrys,ntaylor,none,error;
    double prxa,xstep,estep;
    double *ryselem,*ryscoef,*talpha,*walpha,*ualpha,*casym;
    Jobtime *jobtime;
  public:
    Ryspoly();
    ~Ryspoly();
    void init();
    int getlength();
    int export(char *dest,int length=0);
    int import(char *source,int length=0);
    float getmemory();
    int makeroots(int anrys);
    double getualpha(int mrys,int mroot,double xa);
    double getwalpha(int mrys,int mroot,double xa);
    double gettalpha(int mrys,int mroot,double xa);
    int getalphas(int mrys,double xa,double *xtalpha,
      double *xwalpha,double *xualpha);
    int getalphas(int mrys,float xa,float *xtalpha,
      float *xwalpha,float *xualpha);
    int saveroots(char *filename,char *usermessage);
  private:
    int makelement();
    int makesubelement(double xa);
    double getelement(int bnrys,double xa);
    double fryspoly(int mrys,double xt);
    double asymptotic(int k,int ma,double *xalpha);
};

#endif //__RYSPOLY_H

