/*
----------------------------------------------------------------------
  C++ classes designed for relaxation energy software package
  Yufeng Chen, Michel A. Van Hove
  Lawrence Berkeley National Laboratory (LBNL), Berkeley, CA 94720
  Copyright (c) Van Hove Group 1997. All rights reserved.
----------------------------------------------------------------------

  Yufeng Chen, LBNL MS 2-100, 1 Cyclotron Road, Berkeley, CA 94720
  Phone (510) 486-4581 Fax (510) 486-5530 Email. ychen@LBL.gov

  M. A. Van Hove, LBNL MS 66-100, 1 Cyclotron Road, Berkeley, CA 94720
  Phone (510) 486-6160 Fax (510) 486-4995 Email. MAVanhove@LBL.gov
----------------------------------------------------------------------

List of classes

1. Twoelec
Description: calculation of expectation value
Constructors and Destructor:
  Twoelec() --- constructor.
Member Functions:
  void init() --- initialization, must call before any other calls
  int getlength() --- get memory size of the object needs
  int export(char *dest,int length) --- export the whole object
  int import(char *source,int length) --- import the whole object
  int getnumpoint() --- get number of non-zero integrals
  int getnumbasis() --- get number of basis functions
  float getmemory() --- get memory size of the object needs
  int loadcurve(char *filename,char *sysname) --- load two-electron
    integral data and molecular coefficients from gaussian output
    file (filename) for the system with name (sysname)
  float getmocoef(int basis,int elevel) --- get molecular coefficient
    of a energy level elevel (0-nbasis*2-1, 0 - nbasis-1 stands for
    alpha electron, nbasis - nbasis*2-1 stands for beta electron) for
    a specified basis (0-nbasis-1)
  float getintegral(int ka) --- get two-electron integral component
    (ka=0 - ndata)
  int getintadd(int ka) --- get index (i j k l) of the two-electron
    integral component (index=i+j*nbasis+k*nbasis*nbasis+l*nbasis*
    nbasis*nbasis)
  int maskinnershell() --- make a mask which indicate the corresponding
    1s inner shell for each electron
  int sumintegral() --- summing all the two-electron integrals
  float oneintegral(int ma,int mb,int ia,int ib,int ic,int id,
    float integral) --- multiply one two-electron integral with
    molecular coefficients. ma and mb are electron identities,
    (ia ib ic id) are two-electron integral indecies, integral is
    the two-electron integral
  int savecurve(char *filename,char *usermessage) --- save the
    summation of all the two-electron integrals for both alpha and
    beta electrons
----------------------------------------------------------------------
*/

#ifndef __TWOELEC_H
#define __TWOELEC_H

#include <time.h>

#include "jobtime.h"

class Twoelec
{ private:
    int datatype,nbasis,nalpha,nbeta,ngauss,ndata,charge,
      multiplicity,relax,error;
    char *sysname,*basisname,*orbital;
    int *twoeadd;
    float *twoeint,*expecval,*eigenval,*mocoef,*innermask;
    Jobtime *jobtime;
  public:
    Twoelec();
    ~Twoelec();
    void init();
    int getlength();
    int export(char *dest,int length);
    int import(char *source,int length);
    int getnumpoint();
    int getnumbasis();
    int getrelax();
    float getmemory();
    int loadcurve(char *finname,char *foutname,char *sysname);
    float getmocoef(int basis,int elevel);
    float getintegral(int ka);
    int getintadd(int ka);
    int maskinnershell();
    int sumintegral();
    float oneintegral(int ma,int mb,int ia,int ib,int ic,int id,
      float integral);
    int savecurve(char *filename,char *usermessage);
};

#endif //__TWOELEC_H

/*
----------------------------------------------------------------------
  char sysname[80] orbital[nbasis*10]
  int twoeadd[ndata]
  float twoeint[ndata] float expecval[nbasis*16]
  float eigenval[nbasis*2] mocoef[nbasis*nbasis*2]
  innermask[nbasis*nbasis*4]
  basis --- basis function 0 -> nbasis-1
  elevel --- energy level 0 -> number of occupied -1
----------------------------------------------------------------------
  two-electron integral index statistics (n2=n*n n4=n*n*n*n)

  index             number            factor    total
  i==j==k==l        n                   1       n
  (i==j) != (k==l)  n(n-1)/2            2       n(n-1)
  (i!=j) == (k!=l)  n(n-1)/2            4       2n(n-1)
  (i==j, k!=l) ||
    (i!=j, k==l)    n2(n-1)/2           4       2n2(n-1)
  (i!=j!=k!=l)      n(n-1)(n2-n-2)/8    8       n(n-1)(n2-n-2)

  total             n(n+1)(n2+n+2)/8            n4
----------------------------------------------------------------------

  The input file is a gaussian output file. To make gaussian
  calculation, use Int=(NoRaff,Nosymm) and Pop=Full command.
  Here is a example of gaussian input file
  -----------------------
  # HF/3-21G Int=(NoRaff,Nosymm) IOp(3/33=4)
      SCF= conv Extralinks=(L316) Pop=Full Test  (one line)

  H2O exchange term

   0   1
  H
  O  1 R1
  H  2 R2 1 A
     Variables:
  R1 1.0
  R2 2.0
  A  105.0
  -----------------------



----------------------------------------------------------------------
*/


